#include <stdio.h>
#include <stdbool.h>
#include "date.h"

static bool checkDate(struct date);
static int compareTo(struct date, struct date);
static int maxDays(int, int);
static bool leapYear(int);
static int n(struct date);

/************************************************************************
 * Richiede di inserire una data e restituisce la corrispondente 
 * struttura. Se la data inserita non e' corretta viene chiesta di nuovo.
 ************************************************************************/
struct date insertDate(void) {
	int day, month, year;
	struct date d;
	bool end;

	do { 
		printf("Inserire una data uguale o superiore al 1/3/1970 nel formato d/m/y.\n");
		scanf("%i/%i/%i", &day, &month, &year);
		d.day = day;
		d.month = month;
		d.year = year;
		end = checkDate(d);
		if (!end)
			printf("La data inserita non e' valida.\n");
	} while (!end);

	return d;
}


/********************************************************************
 * Verifica se la data è corretta.
 ********************************************************************/
static bool checkDate(struct date d) {
	int n;
	static struct date min_date = {1, 3, 1700};

	// anno 4 cifre
	if (d.year < 999 || d.year > 9999)
		return false;
	
	// mese compreso tra 1 e 12
	if (d.month < 1 || d.month > 12) 
		return false;
  	
  	// giorno compreso tra 1 e numero massimo giorni del mese
	n = maxDays(d.month, d.year);
	if (d.day < 1 || d.day > n)
 		return false;
	
	// data successiva a marzo 1700
	if (compareTo(d, min_date) < 0)
		return false;
  
	return true;
}


/*******************************************************************
 * Confronta le due date this e that
 * Se this e' seguente a that restituisce un valore positivo
 * Se this coincide con that restituisce 0
 * Se this e' antecedente a that restituisce un valore negativo
 *******************************************************************/
int compareTo(struct date this, struct date that) {
	int diff = this.year - that.year;
	if (diff != 0)
		return diff;

	// gli anni sono gli stessi
	diff = this.month - that.month;
	if (diff != 0)
		return diff;

	// gli anni e i mesi sono gli stessi
	return this.day - that.day;
}


/********************************************************************
 * Restituisce il numero massimo di giorni per il mese e l'anno passati
 * come parametro
 ********************************************************************/
static int maxDays(int month, int year) {
	if (month == 1 || month == 3 || month == 5 || month == 7 || 
		month == 8 || month == 10 || month == 12)
		return 31;
	else if( month == 4 || month == 6 || month == 9 || month == 11 )
		return 30;
	return 28 + (leapYear(year) ? 1 : 0); // ultima possibilita': febbraio
}


/********************************************************************
 * Restituisce true se l'anno e' bisestile, false altrimenti
 ********************************************************************/
static bool leapYear(int year) {
	if (year % 4 == 0 && (year % 100 != 0 || year % 400 == 0))
		return true;
	return false;
}

/********************************************************************
 * Calcola la differenza tra due date
 * utilizzando la funzione di supporto n()
 ********************************************************************/
int elapsedDays(struct date d1, struct date d2) {
	int n1, n2;
	n1 = n(d1);
	n2 = n(d2);

	return n2 - n1;
}

static int n(struct date d) {
	static struct date marzo_1900 = {1, 3, 1900};
	static struct date marzo_1800 = {1, 3, 1800};
	int n, f, g, k;
  
	if (d.month <= 2) {
		f = d.year - 1;
		g = d.month + 13;
	} else {
		f = d.year;
		g = d.month + 1;
	}
  
	if (compareTo(d, marzo_1900) >= 0) // data successiva a 1/3/1900
		k = 0; 
	else if (compareTo(d, marzo_1800) >= 0) // data compresa tra 1/3/1800 e 28/2/1900
		k = 1; 
	else // data compresa tra 1/3/1700 e 28/2/1800
		k = 2; 

	n = 1461 * f / 4 + 153 * g / 5 + d.day + k;
}

