#include <stdio.h>
#include "punto.h"
#include "rettangolo.h"


/********************************************************************
 * Riceve due punti a, b e restituisce il rettangolo con i lati paralleli 
 * agli assi cartesiani che ha il segmento ab come diagonale. 
 ********************************************************************/
struct rettangolo crea_rettangolo(struct punto a, struct punto b) {
	struct rettangolo r;

	if (a.ascissa < b.ascissa) {
		r.max.ascissa = b.ascissa;
		r.min.ascissa = a.ascissa;
	} else {
		r.max.ascissa = a.ascissa;
		r.min.ascissa = b.ascissa;
	}
	if (a.ordinata < b.ordinata) {
		r.max.ordinata = b.ordinata;
		r.min.ordinata = a.ordinata;
	} else { 
		r.max.ordinata = a.ordinata;
		r.min.ordinata = b.ordinata;
	}

	return r;
}


/********************************************************************
 * Riceve un rettangolo e stampa il suo perimetro sul piano cartesiano
 ********************************************************************/
void stampa_perimetro(struct rettangolo r) {
	// utilizziamo una matrice di supporto
	// che rappresenta la regione di piano ammissibile
	// IDEA: disegniamo prima il perimetro del rettangolo sulla matrice
	// e poi stampiamo la matrice a video.
	char piano [DIM] [DIM];
	int i, j;

	// inizializziamo la matrice
	for (i = 0; i  < DIM; i++)
		for (j = 0; j < DIM; j++)
			piano[i][j] = ' ';
	
	// tracciamo i lati paralleli all'asse x
	for (i = r.min.ascissa; i <= r.max.ascissa; i++) {
		piano[r.min.ordinata][i] = '*';
		piano[r.max.ordinata][i] = '*';
	}

	// tracciamo i lati paralleli all'asse y
	for (i = r.min.ordinata; i <= r.max.ordinata; i++) {
		piano[i][r.min.ascissa] = '*';
		piano[i][r.max.ascissa] = '*';
	}

	// stampiamo la matrice a video
	// man mano inseriamo le unita' di misura sull'asse y
	for (i = DIM - 1; i  >= 0; i--) {
		printf("%i", i);
		for (j = 0; j < DIM; j++)
			printf(" %c", piano[i][j]);
		printf("\n");
	}
	// stampiamo le unita' di misura sull'asse x
	printf(" ");
	for (j = 0; j < DIM; j++)
		printf(" %i", j);
	printf("\n");

}


/********************************************************************
 * Stampa alternativa del perimetro sul piano cartesiano,
 * senza l'uso di una matrice di supporto.
 * La stampa scorren le ordiate, dalla maggiore alla minore.
 ********************************************************************/
void stampa_perimetro_variante (struct rettangolo r) {
	int i, j;

	// stampiamo la zona superiore in cui non appare il rettangolo
	for (i = DIM - 1; i > r.max.ordinata; i--) {
		printf("%i", i);
		for (j = 0; j < DIM; j++)
			printf("  ");
			printf("\n");
	}
	
	// stampiamo il lato orrizontale superiore
	printf("%i", r.max.ordinata);
	for (j = 0; j < r.min.ascissa;  j++)
		printf("  ");
	for ( ; j <= r.max.ascissa;  j++)
		printf(" *");
	for ( ; j < DIM; j++)
		printf("  ");
	printf("\n");
	
	// stampiamo gli eventuali lati verticali
	for (i = r.max.ordinata - 1; i  > r.min.ordinata; i--) {
		printf("%i", i);
		for (j = 0; j < DIM;  j++)
			if (j == r.min.ascissa || j == r.max.ascissa)
				printf(" *");
			else
				printf("  ");
		printf("\n");
	}

	// se r.max.ordinata == r.min.ordinata 
	// abbiamo gia' finito di stampare il rettangolo
	// altrimenti stampiamo il lato orrizontale inferiore
	if (r.max.ordinata != r.min.ordinata) {
		printf("%i", r.min.ordinata);
		for (j = 0; j < r.min.ascissa;  j++)
			printf("  ");
		for ( ; j <= r.max.ascissa;  j++)
			printf(" *");
		for ( ; j < DIM; j++)
			printf("  ");
		printf("\n");
	}

	// stampiamo la zona inferiore in cui non appare il rettangolo
	for (i = r.min.ordinata - 1; i >=0; i--) {
		printf("%i", i);	
		for (j = 0; j < DIM; j++)
			printf("  ");
		printf("\n");
	}
  
	// stampiamo le unita' di misura sull'asse x
	printf(" ");
	for (j = 0; j < DIM; j++)
		printf(" %i", j);
	printf("\n");  

}
